%
% VistaRestoreTools1.0
% IMAGE DENOISING DEMO USING WAVELET-BASED ALGORITHMS
%
% In this demo, different wavelet-based denoising algorithms 
% based on different assumptions on natural images statistics
% are compared in denoising problems using (1) high variance white
% Gaussian noise, and (2) JPEG noise (which is non-Gaussian, colored
% and signal dependent). Classical algorithms require the knowledge
% of variance noise. In the case of unknown variance, it is estimated by
% 'function_stdEst2D.m' (see Foi06 for details).
%
% The original image is corrupted by using 'apply_degradation_2.m'
% The compared denoising algorithms and the corresponding Matlab functions are:
%
%   * Hard Thresholding [Donoho95,Simoncelli99].... hardthres.m
%       Assumes the noise variance is known.
%       Applies the threshold=3*sigma_n rule.
%
%   * Soft Thresholding [Donoho95,Simoncelli99].... softthres.m
%       Assumes the noise variance is known.
%       Applies the threshold=3*sigma_n rule.
%
%   * Bayesian denoising assuming (1) Gaussian noise of known variance 
%     and, (2) Gaussian marginals for the wavelet coefficients of natural 
%     images [Figueiredo01]........................ bayesian_gauss_margin.m
%
%   * Kernel regularization in the wavelet domain with kernels inspired in
%     mutual information measures [Laparra08] ..... mi_kernel_denoising.m
%
%     The algorithm [Laparra08] requires some knowledge on the signal and 
%     the noise:
%     1- Average variance of the signal in the wavelet domain to set
%        the SVM penalization profile, C_i. This variance profile is an
%        intrinsic feature of natural images and can be estimated from 
%        a natural image database. 
%     2- Average standard deviation of the noise in the wavelet domain to
%        set the insensitivity profile, epsilon_i. This deviation profile 
%        can be estimated from noise samples.
%     3- 2D histogram of the noise in the spatial domain to look for the
%        SVM parameters that better identify the noise. SVM parameter
%        optimization can be done by minimizing the KL divergence between
%        the known noise histogram and the histogram of the estimated
%        noise.
%     The code provided in VistaRestoreTools1.0 skips all the off-line 
%     preprocessing steps:
%     1- Average variance of natural images in the steerable domain was
%        precomputed from the McGill natural image data base 
%        (http://tabby.vision.mcgill.ca/) and stored in the file: 
%                "C_profile.mat"
%     2- Average standard deviation profile of Gaussian noise and 
%        JPEG noise was precomputed from (1) noise realizations and 
%        (2) JPEG compressed (at MATLAB quality compression Q=7) examples 
%        respectively, and stored in the files:
%    		"Epsilon_profile_Gaussian.mat"
%    		"Epsilon_profile_JPEG_Q7.mat"
%        If the Q factor of the JPEG quantization is modified, or another noise 
%        source is present, the epsilon profile should be computed by looking at
%        the average wavelet transform of noise samples.
%        For signal independent noise sources, the unit variance profile
%        scales as: epsilon_profile=sqrt(Var)*unit_variance_epsilon_profile..
%        Therefore, for white noise Gaussian sources the code provided here
%        is general assuming known variance (as the other algorithms).
%	  3- KLD-based SVR selection is not implemented in the code. Instead, fixed
%        (near optimum) values are given in each case (Gaussian and JPEG), in the 
%        ranges described in [Laparra08].
%
% [Laparra08]    Laparra, Gutierrez, Camps & Malo. 
%                Recovering Wavelet Relations using SVM for Image Denoising
%                IEEE Int. Conf. Im. Proc., San Diego, 2008
%
% [Donoho95]     David L. Donoho and Iain M. Johnstone. Adapting to unknown 
%                smoothness via wavelet shrinkage. J. Am. Stat. Assoc., 90:12001224, 1995.
%
% [Simoncelli99] E. Simoncelli. Bayesian denoising of visual images in the wavelet 
%                domain. In Bayesian Inference in Wavelet Based Models, pages 291308. 
%                Springer-Verlag, NY, 1999.
%
% [Figueiredo01] M. Figueiredo and R. Nowak. Wavelet-based image estimation: 
%                an empirical Bayes approach using Jeffreys noninformative prior. 
%                IEEE Transactions on Image Processing, 10(9):13221331, 2001.
%
% [Simoncelli97] E. Simoncelli. MatlabPyrTools. Matlab toolbox for wavelet transforms 
%                http://www.cns.nyu.edu/~lcv/software.php
%
% [Foi06]        Alessandro Foi. Routine  'function_stdEst2D.m'  for automatic
%                noise variance estimation. Tampere University of
%                Technology - 2005-2006
%                http://www.cs.tut.fi/~lasip/2D

clear all

% Original Image
% ---------------
load barbara256

% Image degradation (see image_degradation_demo.m)
% ------------------------------------------------
%
%     In the following loop over degradations select one of the following degradations (or both):
%
%     (1) White Gaussian noise
%     (2) JPEG noise
%        

% Image distortion measures
% -------------------------
%
% If you have installed the following subjectively meaningful distortion measures:
%   * VistaQualityTools 
%      http://www.uv.es/vista/vistavalencia/software/software.html
%   * Structural Similarity Index
%      http://www.cns.nyu.edu/~lcv/software.php
%   * S-CIELAB
%      http://white.stanford.edu/~brian/scielab/
% set the next 'meaningful_measures' variable to 1 to evaluate the distortion and
% the restoration results. By default meaningful_measures=0 (and just PSNR
% is given).

meaningful_measures=0;

for ii=1:2 % Loop over degradations

degradation=ii   

if degradation==1

    %
    % Almost denoising problem
    %     
        deg_model='blur_wt_gn';
        cut_off_blur=27.5/32;  % 32cpd is assumed to be the Nyquist frequency. 50 cpd is the cut-off frequency: no blur at all
        min_val_blur=0.05;
        variance=200;        % severe noise
        [Id1,PSF1] = apply_degradation_2(Im,deg_model,cut_off_blur,min_val_blur,variance);

        Id=Id1;
        PSF=PSF1;

        PSNR=psnr(Im,Id);
        if meaningful_measures==1
        % For subjectively meaningful distortion measures, please install: 
        %   * VistaQualityTools 
        %      http://www.uv.es/vista/vistavalencia/software/software.html
        %   * Structural Similarity Index
        %      http://www.cns.nyu.edu/~lcv/software.php
        %   * S-CIELAB
        %      http://white.stanford.edu/~brian/scielab/
        %
        ssim=ssim_index(Im,Id);
        d=distance_wav_color(Im,Id);
        imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
        imr(:,:,1)=Id;imr(:,:,2)=Id;imr(:,:,3)=Id;
        SCIELab = calculo_scielab(imm,imr,1);

        figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,title(['White Gaussian Noise (\sigma^2=400)']),
        %xlabel(['PSNR=',num2str(PSNR)])
        xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   SSIM=',num2str(round(100*ssim)/100)],['SCIELab=',num2str(round(100*SCIELab)/100),'  d_{wav}=',num2str(round(100*d)/100)]))

        else
        figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,title(['White Gaussian Noise (\sigma^2=400)']),
        xlabel(['PSNR=',num2str(PSNR)])
            
        end    
else    
    deg_model='jpeg';
    Q=7;
    [Id5,PSF5] = apply_degradation_2(Im,deg_model,Q);
    Id=Id5;
    PSF=PSF5;
    
        PSNR=psnr(Im,Id);
        if meaningful_measures==1
        % For subjectively meaningful distortion measures, please install: 
        %   * VistaQualityTools 
        %      http://www.uv.es/vista/vistavalencia/software/software.html
        %   * Structural Similarity Index
        %      http://www.cns.nyu.edu/~lcv/software.php
        %   * S-CIELAB
        %      http://white.stanford.edu/~brian/scielab/
        %
        ssim=ssim_index(Im,Id);
        d=distance_wav_color(Im,Id);
        imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
        imr(:,:,1)=Id;imr(:,:,2)=Id;imr(:,:,3)=Id;
        SCIELab = calculo_scielab(imm,imr,1);

        figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,title('JPEG Noise (Q=7)'),
        %xlabel(['PSNR=',num2str(PSNR)])
        xlabel(char(['  PSNR=',num2str(round(10*PSNR)/10),'   SSIM=',num2str(round(100*ssim)/100)],['SCIELab=',num2str(round(100*SCIELab)/100),'  d_{wav}=',num2str(round(100*d)/100)]))

        else
        figure,colormap gray,imshow(Id(33:end,33:end)/255),axis square,title('JPEG Noise (Q=7)'),
        xlabel(['PSNR=',num2str(PSNR)])
            
        end    
end 


% Image Denoising (see hardthres.m, softthres.m, bayesian_gauss_margin.m, mi_kernel_denoising.m)
% ----------------------------------------------------------------------------------------------
%

tic

%% HARD THRESHOLDING

% Known variance or variance estimation
if degradation==1
   va=variance; 
else        % Variance estimation for the JPEG case
   dev = function_stdEst2D(Id);
   va=dev^2;
end
disp('Hard Thresholding Image Denoising ...')
imr_hard = hardthres(Id,va);
PSNR=psnr(Im,imr_hard);

if meaningful_measures==1
% For subjectively meaningful distortion measures, please install: 
%   * VistaQualityTools 
%      http://www.uv.es/vista/vistavalencia/software/software.html
%   * Structural Similarity Index
%      http://www.cns.nyu.edu/~lcv/software.php
%   * S-CIELAB
%      http://white.stanford.edu/~brian/scielab/
%
ssim=ssim_index(Im,imr_hard);d=distance_wav_color(Im,imr_hard);
imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
imr(:,:,1)=imr_hard;imr(:,:,2)=imr_hard;imr(:,:,3)=imr_hard;
SCIELab = calculo_scielab(imm,imr,1);
dis_hard=[PSNR ssim d SCIELab];
else
dis_hard=[PSNR];
end

%% SOFT THRESHOLDING

% Known variance or variance estimation
if degradation==1
   va=variance; 
else        % Variance estimation for the JPEG case 
   dev = function_stdEst2D(Id);
   va=dev^2;
end
disp('Soft Thresholding Image Denoising ...')
imr_soft = softthres(Id,va);
PSNR=psnr(Im,imr_soft);
if meaningful_measures==1
% For subjectively meaningful distortion measures, please install: 
%   * VistaQualityTools 
%      http://www.uv.es/vista/vistavalencia/software/software.html
%   * Structural Similarity Index
%      http://www.cns.nyu.edu/~lcv/software.php
%   * S-CIELAB
%      http://white.stanford.edu/~brian/scielab/
%
ssim=ssim_index(Im,imr_soft);d=distance_wav_color(Im,imr_soft);
imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
imr(:,:,1)=imr_soft;imr(:,:,2)=imr_soft;imr(:,:,3)=imr_soft;
SCIELab = calculo_scielab(imm,imr,1);
dis_soft=[PSNR ssim d SCIELab];
else
dis_soft=[PSNR];    
end

%% BAYESIAN DENOISING 

% Known variance or variance estimation
if degradation==1
   va=variance; 
else        % Variance estimation for the JPEG case
   dev = function_stdEst2D(Id);
   va=dev^2;
end
disp('Bayesian Image Denoising assuming Gaussian signal and noise...')
imr_bayes = bayesian_gauss_margin(Id,va);
PSNR=psnr(Im,imr_bayes);
if meaningful_measures==1
% For subjectively meaningful distortion measures, please install: 
%   * VistaQualityTools 
%      http://www.uv.es/vista/vistavalencia/software/software.html
%   * Structural Similarity Index
%      http://www.cns.nyu.edu/~lcv/software.php
%   * S-CIELAB
%      http://white.stanford.edu/~brian/scielab/
%
ssim=ssim_index(Im,imr_bayes);d=distance_wav_color(Im,imr_bayes);
imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
imr(:,:,1)=imr_bayes;imr(:,:,2)=imr_bayes;imr(:,:,3)=imr_bayes;
SCIELab = calculo_scielab(imm,imr,1);
dis_bayes=[PSNR ssim d SCIELab];
else
dis_bayes=[PSNR];    
end    

%% KERNEL DENOISING

% Parameter settings: epsilon_profile and (epsilon,C,S) values
if degradation==1
     C       = 3000;
     S       = 4;
     epsilon = 2;
     load Epsilon_profile_Gaussian.mat
     epsilon_profile  = perfil_eps*sqrt(variance);    % Epsilon profile depends on the (known) noise variance
elseif degradation==2     
     C       = 3000;
     S       = 1;
     epsilon = 1.7;  
     load Epsilon_profile_JPEG_Q7.mat
     epsilon_profile  = perfil_eps;
end

% Restore image
disp('Mutual Info Kernel Image Denoising ...')
imr_kernel = mi_kernel_denoising(Id,epsilon,C,S,epsilon_profile);

PSNR=psnr(Im,imr_kernel);
if meaningful_measures==1
% For subjectively meaningful distortion measures, please install: 
%   * VistaQualityTools 
%      http://www.uv.es/vista/vistavalencia/software/software.html
%   * Structural Similarity Index
%      http://www.cns.nyu.edu/~lcv/software.php
%   * S-CIELAB
%      http://white.stanford.edu/~brian/scielab/
%
ssim=ssim_index(Im,imr_kernel);d=distance_wav_color(Im,imr_kernel);
imm(:,:,1)=Im;imm(:,:,2)=Im;imm(:,:,3)=Im;
imr(:,:,1)=imr_kernel;imr(:,:,2)=imr_kernel;imr(:,:,3)=imr_kernel;
SCIELab = calculo_scielab(imm,imr,1);
dis_kernel=[PSNR ssim d SCIELab];
else
dis_kernel=[PSNR];    
end    
disp('Done!')

dis(:,:,ii)=[dis_hard;dis_soft;dis_bayes;dis_kernel];

if meaningful_measures==1
% If you installed VistaQualityTools, SSIM and S-CIELab       
%       
figure,subplot(221),imshow(imr_hard(33:end,33:end)/255),title('Hard'),xlabel(char(['  PSNR=',num2str(round(10*dis(1,1,ii))/10),'   SSIM=',num2str(round(100*dis(1,2,ii))/100)],['SCIELab=',num2str(round(100*dis(1,4,ii))/100),'  d_{wav}=',num2str(round(100*dis(1,3,ii))/100)]))
       subplot(222),imshow(imr_soft(33:end,33:end)/255),title('Soft'),xlabel(char(['  PSNR=',num2str(round(10*dis(2,1,ii))/10),'   SSIM=',num2str(round(100*dis(2,2,ii))/100)],['SCIELab=',num2str(round(100*dis(2,4,ii))/100),'  d_{wav}=',num2str(round(100*dis(2,3,ii))/100)]))
       subplot(223),imshow(imr_bayes(33:end,33:end)/255),title('Bayesian with Gaussian Marginals'),xlabel(char(['  PSNR=',num2str(round(10*dis(3,1,ii))/10),'   SSIM=',num2str(round(100*dis(3,2,ii))/100)],['SCIELab=',num2str(round(100*dis(3,4,ii))/100),'  d_{wav}=',num2str(round(100*dis(3,3,ii))/100)]))
       subplot(224),imshow(imr_kernel(33:end,33:end)/255),title('Kernel'),xlabel(char(['  PSNR=',num2str(round(10*dis(4,1,ii))/10),'   SSIM=',num2str(round(100*dis(4,2,ii))/100)],['SCIELab=',num2str(round(100*dis(4,4,ii))/100),'  d_{wav}=',num2str(round(100*dis(4,3,ii))/100)]))
else
figure,subplot(221),imshow(imr_hard(33:end,33:end)/255),title('Hard'),xlabel(char(['  PSNR=',num2str(round(10*dis(1,1,ii))/10)]));
       subplot(222),imshow(imr_soft(33:end,33:end)/255),title('Soft'),xlabel(char(['  PSNR=',num2str(round(10*dis(2,1,ii))/10)]));
       subplot(223),imshow(imr_bayes(33:end,33:end)/255),title('Bayesian with Gaussian Marginals'),xlabel(char(['  PSNR=',num2str(round(10*dis(3,1,ii))/10)]));
       subplot(224),imshow(imr_kernel(33:end,33:end)/255),title('Kernel'),xlabel(char(['  PSNR=',num2str(round(10*dis(4,1,ii))/10)]));
end


toc

end