%
%   RESTORE: image denoising or deconvolution by local regularization.
%
%   RESTORE represents a local regularization framework in which no 
%   assumption is made on the noise nature (in particular, its variance 
%   may be unknown): relative weight of the regularization term is 
%   obtained by using the L-curve method [Hansen93]. The local 
%   implementation of the regularization implies that the noise may 
%   be non-stationary.
%
%   In deconvolution problems, RESTORE needs the PSF information. 
%   Even though the formulation is local, in the current VistaRestoreTools
%   version, the PSF is assumed to be spatially invariant.
%   
%   The restoration ability relies on the ability of the regularization
%   functional to capture/extract/preserve the relevant features of the
%   signal. A number of fixed and adaptive regularization functionals are
%   provided in this function. (see J. Gutierrez et al. IEEE Tr.Im.Proc. 2006
%   for details).
%
%   SYNTAX:
%
%        im_r=restore(im_d,'regulariz_functional',PSF,lambdas);
%
% Output:
% -------
%  * im_r..................= restored image
%
% Input:
% ------
%  * im_d..................= degraded image
%
%  * 'regulariz_functional'= string containing the selected regularization
%                            functional. The available possibilities include:
%              
%          * Classical regularization:
%             - 'L2'.................: Second derivative regularization [Tikhonov79]
%             - 'AR4', 'AR8', 'AR12'.: Wiener filtering with Auto-Regressive (AR) power 
%                                      spectrum estimation. The number in the string 
%                                      indicates the order of the AR model 
%                                      [Banham&Katsaggelos97]
%             - 'CSF'................: Regularization using the inverse of the Contrast
%                                      Sensitivity Function (CSF) [Hunt75].  
%
%          * Regularization based on non-linear perception models
%             - 'PER'................: Divisive normalization regularization functional 
%                                      [Gutierrez06].
% 
%  * PSF....................= Point Spread Function as given by apply_degradation_2 
%                             
%  * lambdas................= Vector with the possible values of the regularization
%                             parameter (relative weight of regularization vs deviation)
%                             The L-curve method is used to select one of
%                             these values for each block of the image.
%
%
% See details at:    J. Gutierrez et al. Regularization Operators for
%                    Natural Images based on Non-linear Perception Models. 
%                    IEEE Trans. Im. Proc. 15(1). 2006
%

function  imr=restaura2(im,method,PSF,lambdas,varargin)

warning('off','MATLAB:dispatcher:InexactMatch')
a=size(im);
bordefil=(ceil(a(1)/16)*16-a(1));
bordecol=(ceil(a(2)/16)*16-a(2));
im_a = [im repmat(im(:,end),1,bordecol); repmat(im(end,:),bordefil,1) repmat(im(end,end),bordefil,bordecol)];
u=0;

%%%%%%

% Effective block size
N=16;

% Block size (there will be overlapping)
M=32;

 if strcmp(method,'AR4')==1
        [imr_a] = regu_modelo_ar4(im_a,PSF,lambdas,N,M,2,0);
 elseif strcmp(method,'AR8')==1
        [imr_a] = regu_modelo_ar8(im_a,PSF,lambdas,N,M,2,0);
 elseif strcmp(method,'AR12')==1
        [imr_a] = regu_modelo_ar12(im_a,PSF,lambdas,N,M,2,0);
 elseif strcmp(method,'L2')==1
        Operador=freqz2(filt2dsegder,M,M);
        [imr_a] = regu_operador(im_a,PSF,lambdas,N,M,Operador,2,0);
 elseif strcmp(method,'CSF')==1
        load CSF_Operator;
        [imr_a] = regu_operador(im_a,PSF,lambdas,N,M,CSF_Operator,2,0);
 elseif strcmp(method,'PER')==1
        corteResp = 1600;
       
        load response_parameters;

        imr_a=regu_perceptual(im_a,PSF,lambdas,N,M,H,k1,k2,corteResp,2,0);
    
 else
        ['The requested method (' method ') is not implemented']
        u=1;
 end
 
 % Set the image to the initial size

 if u==1
    imr=0;  
 else
    imr=imr_a(1:a(1),1:a(2));
 end
