
% MI_KERNEL_DENOISING image denoising by Mutual Info Kernel regularization
% in the steerable wavelet domain.
%
% Regularization in the wavelet domain is made by Support Vector Machine Regression 
% (SVR). SVR needs the specification of:
% (1) The relative SVR insensitivity profile for each wavelet coefficient 
%     'epsilon_profile' and a global factor, 'epsilon', to scale this profile.
% (2) The penalization profile for each wavelet coefficient 'C_profile' and
%     a global factor, 'C', to scale this profile.
% (3) The SVR kernel. In this method the kernel is an anisotropic Gaussian
%     kernel inspired in the relations among natural images wavelet coefficients 
%     in the steerable wavelet domain (measured using mutual information).
%     A global factor, 'S', to scale the width of the kernel is needed.
%
% In order to set these parameters the algorithm requires some knowledge on 
% the signal and the noise:
%     1- Average variance of the signal in the wavelet domain to set
%        the penalization profile, C_profile. This variance profile is an
%        intrinsic feature of natural images and can be estimated from 
%        a natural image database. 
%     2- Average standard deviation of the noise in the wavelet domain to
%        set the insensitivity profile, epsilon_profile. This deviation profile 
%        can be estimated from noise samples.
%     3- 2D histogram of the noise in the spatial domain to look for the
%        SVM parameters (epsilon, C, S) that better identify the noise. 
%        SVM parameter optimization can be done by minimizing the
%        KL divergence between the known/(pre-estimated) noise histogram and 
%        the histogram of the estimated noise.
%     The code provided in VistaRestoreTools1.0 skips all the off-line 
%     preprocessing steps:
%     1- Average variance of natural images in the steerable domain was
%        precomputed from the McGill natural image data base 
%        (http://tabby.vision.mcgill.ca/) and stored in the file: 
%                "C_profile.mat"
%     2- Average standard deviation profile of Gaussian noise and 
%        JPEG noise was precomputed from (1) Gaussian noise realizations and 
%        (2) JPEG compressed (at MATLAB quality compression Q=7) examples 
%        respectively, and stored in the files:
%    		"Epsilon_profile_Gaussian.mat"
%    		"Epsilon_profile_JPEG_Q7.mat"
%        If the Q factor of the JPEG quantization is modified, or another noise 
%        source is present, the epsilon profile should be computed by looking at
%        the average wavelet transform of noise samples.
%        For signal independent noise sources, the unit variance epsilon profile
%        scales as: epsilon_profile=sqrt(Var)*unit_variance_epsilon_profile.
%        Therefore, for white noise Gaussian sources the code provided here
%        is general assuming known variance (as in the other classical algorithms 
%        included in VistaRestoreTools).
%	  3- KLD-based SVR selection is not implemented in the code. Instead, 
%        the user has to specify the parameters (epsilon,C,S).
%        See image_denoising_demo.m for examples on how to set (epsilon,C,S)
%
% SYNTAX:
%
%     im_r = mi_kernel_denoising(im_n,epsilon,C,S,epsilon_profile);
%
% Input
% -----
%
%    * im_n    = Noisy Image
%    * epsilon = scale factor to be applied to the insensitivity
%                'epsilon_profile'
%    * C       = scale factor to be applied to the penalization 
%                'C_profile'
%    * S       = scale factor to be applied to the width of the Kernel
%    * epsilon_profile = insensitivity profile (as in
%               Epsilon_profile_Gaussian.mat or
%               Epsilon_profile_JPEG_Q7.mat)
%
% Output
% ------
%
%    * im_r    = restored image
%
% Notes
% ----- 
%	- Steerable wavelet transform, 8 orientations, 4 scales (needs MatlabPyrTools)
%	- Only works for 256x256 images
%
% REFERENCES:
%
% [Laparra08] Laparra, Gutierrez, Camps & Malo. 
%             Recovering Wavelet Relations using SVM for Image Denoising
%             IEEE Int. Conf. Im. Proc., San Diego, 2008
%
% [Simoncelli97] E. Simoncelli. MatlabPyrTools. Matlab toolbox for wavelet transforms 
%             http://www.cns.nyu.edu/~lcv/software.php

function IMM=mi_kernel_denoising(Im2,epsilon,Csvm,sigma,perfil_eps)


load C_profile
perfil_sigma=[1 1 1 1];

sigma_2 = sigma;
sigma_1 = sigma_2/2;
AAlfa=linspace(-90,90,9);

for n=1:8
    Kernel_M(:,:,n) = kernelgauss2Dsteerable(16,sigma_2,sigma_1,AAlfa(n));
end

perfil_eps_M = perfil_eps*epsilon;
perfil_C_M = perfil_C*Csvm;

ESCALAS=4;
ORIENTACIONES=7;
[pyrR,ind]=buildSFpyr(Im2,ESCALAS,ORIENTACIONES);

L=(1:256)';
UNOS=ones(1,256)';
pyr2=pyrR;
for esc=1:ESCALAS
    for or=1:ORIENTACIONES+1
	fprintf('.')
        BAND_NUM=1+or+(esc-1)*(ORIENTACIONES+1);
        INDD=pyrBandIndices(ind,BAND_NUM);
        
        RES = pyrBand(pyrR, ind, BAND_NUM);
        KB=Kernel_M(:,:,or).^(perfil_sigma(esc));
       
        P_C = perfil_C_M(INDD(1));       
        P_E = perfil_eps_M(INDD(1));
        
        R=blkproc(RES,[16 16],'SVRegression(x,P1,P2,P3,P4,P5)',L,KB,P_C,P_E,UNOS);
        
        pyr2(INDD)=R(:);
       
    end
end
pyr2(1:256*256)=zeros;
IMM=reconSFpyr(pyr2,ind);

fprintf('\n')
