function A=softthres(Im2,var,varargin)

% SOFTTHRES image denoising by soft thresholding in the wavelet domain.
%
% This routine uses 4 scales QMF orthogonal wavelets (see MatlabPyrTools).
% By default, threshold is derived from the standard deviation of the noise
% using values optimized to minimize MSE in a set of natural images.
% Precomputed threshold values are given for variances in the range
% [0,1600].
% The user may specify a different threshold factor, i.e.
% thres=thres_factor*sigma_n.
%
% Soft thresholding [Donoho95] can be derived in a Bayesian framework using 
% a very specific combination of noise and image models (MAP estimation of
% Generalized Laplacian PDF signal with particular kurtosis (k=1) assuming 
% Gaussian noise [Simoncelli99]).
% 
% SYNTAX:
% 
%       im_r = softthres(im_n,variance,thres_factor);
%
%  Input
%  -----
%    * im_n         = noisy image
%    * variance     = noise variance
%    * thres_factor = factor on the noise deviation to set the threshold (optional!)
%                     If no factor is provided, threshold is estimated 
%  Output
%  ------
%    * im_r         = denoised image 
% 
% REFERENCES:
%
% [Donoho95]     David L. Donoho and Iain M. Johnstone. Adapting to unknown 
%                smoothness via wavelet shrinkage. J. Am. Stat. Assoc., 90:12001224, 1995.
% [Simoncelli99] E. Simoncelli. Bayesian denoising of visual images in the wavelet 
%                domain. In Bayesian Inference in Wavelet Based Models, pages 291308. 
%                Springer-Verlag, NY, 1999.
% [Simoncelli97] E. Simoncelli. MatlabPyrTools. Matlab toolbox for wavelet transforms 
%                http://www.cns.nyu.edu/~lcv/software.php

[pyr,ind]=buildwpyr(Im2,4);

if nargin==2
   if var<=1600
      fun=[1,4,10,17,25,34,44,55,66;0,5,10,15,20,25,30,35,40];
      des=sqrt(var);
      t=interp1(fun(2,:),fun(1,:),des);
      
      s = abs(pyr) - t;
      s = (s + abs(s))/2;
      y = sign(pyr).*s;

      PYR2=y;
      PYR2(65281:65536)=pyr(65281:65536);

      A=reconwpyr(PYR2,ind);
      
   else
      disp('  Input variance exceeds the precomputed range for')      
      disp('  optimal threshold estimation.')
      disp('  Please specify threshold factor on variance!')
      A=0;
   end
else
   t=varargin{1}*sqrt(var);
   s = abs(pyr) - t;
   s = (s + abs(s))/2;
   y = sign(pyr).*s;

   PYR2=y;
   PYR2(65281:65536)=pyr(65281:65536);

   A=reconwpyr(PYR2,ind);
end